#version 330 core
struct PointLight
{
	vec3 position;
	vec3 diffuse;
	//attenuation
	float constant;
	float linear;
	float quadratic;
};
in vec3 FragmentPosition;
in vec2 TexCoords;
in mat3 TBN;
in vec3 Normal;

layout (location = 0) out vec3 gPosition;
layout (location = 1) out vec3 gNormal;
layout (location = 2) out vec4 gAlbedo;

#define NR_OF_POINT_LIGHTS 32
uniform sampler2D texture_diffuse;
uniform sampler2D texture_normal;
uniform samplerCube depthMap;

uniform int useNormalMap;
uniform float ambientTerm;
uniform PointLight pointlights[NR_OF_POINT_LIGHTS];
uniform int lightCount;

uniform float farPlane;

uniform mat4 view;

float ShadowCalculation(vec3 fragPos, vec3 lightPos )
{
	
	vec3 fragToLight = fragPos - lightPos;
	    
	float closestDepth = texture(depthMap, fragToLight).r;
	
	closestDepth *= farPlane;
	
	float currentDepth = length(fragToLight);
		
	float shadow  = 0.0;
	float bias    = 0.2 ;
	float samples = 3.0;
	float offset  = 0.1;
	for(float x = -offset; x < offset; x += offset / (samples * 0.5))
	{
		for(float y = -offset; y < offset; y += offset / (samples * 0.5))
		{
			for(float z = -offset; z < offset; z += offset / (samples * 0.5))
			{
				float closestDepth = texture(depthMap, fragToLight + vec3(x, y, z)).r; 
				closestDepth *= farPlane;   
				if(currentDepth - bias > closestDepth)
					shadow += 1.0;
			}
		}
	}
	shadow /= (samples * samples * samples);

    return shadow;
}
void main()
{    

	vec3 normal;
	if(useNormalMap == 1){
	normal = texture(texture_normal, TexCoords).rgb;
	normal = normalize(normal * 2.0 -1.0);
	normal = normalize(TBN * normal);
	}
	else{
	normal = normalize(Normal);
	}
	

	vec4 color = texture(texture_diffuse, TexCoords);
	vec3 ambient = ambientTerm * color.xyz;

	vec3 diffuseIntensitiy = vec3(0.0f,0.0f,0.0f);
	for(int i = 0; i < NR_OF_POINT_LIGHTS && i < lightCount; i++){
		vec3 lightDirection = normalize(pointlights[i].position - FragmentPosition);
		float diff = max(dot(normal, lightDirection), 0.0f);
		float dist = length(pointlights[i].position - FragmentPosition);
		float attenuation = 1.0f / (pointlights[i].constant + pointlights[i].linear * dist + pointlights[i].quadratic * (dist * dist));    
		float shadow = ShadowCalculation(FragmentPosition,pointlights[i].position);
		if(i==0)
		diffuseIntensitiy += (1.0 - shadow) * (pointlights[i].diffuse * diff * attenuation) ;
		else
		diffuseIntensitiy += (pointlights[i].diffuse * diff * attenuation) ;
	}
	vec3 diffuse = diffuseIntensitiy * color.xyz;
	
    gAlbedo = vec4((ambient + diffuse).xyz, color.a);
	gNormal = (transpose(inverse(view)) * vec4(Normal, 1.0f)).xyz;
	gNormal = normalize(gNormal);
	gPosition = (view * vec4(FragmentPosition, 1.0f)).xyz;
}

