#version 330 core
layout (location = 0) in vec3 position;
layout (location = 1) in vec3 normal;
layout (location = 2) in vec2 texCoords;
layout (location = 3) in vec3 tangent;

out vec3 FragmentPosition;
out vec2 TexCoords;
out mat3 TBN;
out vec3 Normal;

uniform mat4 model;
uniform mat4 view;
uniform mat4 projection;

void main()
{
    gl_Position = projection * view * model * vec4(position, 1.0f);
	FragmentPosition = vec3(model * vec4(position, 1.0f));
	
	mat3 normalMatrix = transpose(inverse(mat3(model)));
	vec3 T = normalize(normalMatrix * tangent);
	vec3 N = normalize(normalMatrix * normal);

	T = normalize(T - dot(T, N) * N);

	vec3 B = cross(N, T);

	TBN = mat3 (T,B,N);
	
	Normal = normalize(normalMatrix * normal);
   
	
	if(abs(Normal.x) > 0.5) {
		TexCoords = FragmentPosition.yz;
	} else if(abs(Normal.z) > 0.5) {
		TexCoords = FragmentPosition.xy;
	} else {
		TexCoords = FragmentPosition.xz;
	}	
	TexCoords *= 0.5;

}