#version 330 core

#define THICKNESS 0.0012
#define ASPECT_RATIO 1.7777777
#define BLUR_RADIUS 2

out vec4 FragColor;

in vec2 TexCoords;

uniform int mode = 0;
//0: normal albedo and ssao
//1: skip ssao
//2: ssao only

uniform sampler2D albedoInput;
uniform sampler2D ssaoInput;

void main() 
{
	if(mode == 1)
		FragColor = vec4(texture(albedoInput, TexCoords).xyz, 1.0f);
	else {
		//blur over BLUR_RADIUS
		vec2 texelSizeInv = 1.0 / vec2(textureSize(ssaoInput, 0));
		float added = 0.0;
		for (int x = -BLUR_RADIUS; x < BLUR_RADIUS; ++x) 
		{
			for (int y = -BLUR_RADIUS; y < BLUR_RADIUS; ++y) 
			{
				vec2 offset = vec2(float(x), float(y)) * texelSizeInv;
				added += texture(ssaoInput, TexCoords + offset).x;
			}
		}
		added /= float((BLUR_RADIUS * 2) * (BLUR_RADIUS * 2));
		
		if(mode == 0)
			FragColor = vec4(texture(albedoInput, TexCoords).xyz * added, 1.0f);
		else
			FragColor = vec4(added,added,added,1.0f);
	}
	
	//crosshair	
	if(TexCoords.x < 0.5+THICKNESS && TexCoords.x > 0.5-THICKNESS
	&& TexCoords.y < 0.5+(THICKNESS * ASPECT_RATIO) && TexCoords.y > 0.5-(THICKNESS * ASPECT_RATIO))
		FragColor.xyz = vec3(1.0)-FragColor.xyz;
	
} 