#version 330 core
out float FragColor;

in vec2 TexCoords;

uniform sampler2D gPosition;
uniform sampler2D gNormal;
uniform sampler2D texNoise;

//max sampleSize 512
uniform vec3 samples[512];

uniform int sampleSize = 64;
uniform float radius = 0.5;
uniform float bias = 0.025;

uniform mat4 projection;
uniform mat4 camera;

uniform vec2 noiseScaleVec;

vec3 toClip(vec3 val) {	
	vec4 calc = vec4(val, 1.0);
	calc = (projection * calc); 
	calc.xyz /= calc.w; 
	return vec3(calc.xyz);
}

void main() {
    vec3 fragPos = (texture(gPosition, TexCoords).xyz);
    vec3 normal = normalize(texture(gNormal, TexCoords).rgb);
    vec3 randomVec = normalize(texture(texNoise, TexCoords * noiseScaleVec).xyz);
    vec3 tangent = normalize(randomVec - normal * dot(randomVec, normal));
    vec3 bitangent = cross(normal, tangent);
    mat3 TBN = mat3(tangent, bitangent, normal);

	float occlusion = 0.0;
	vec3 samplePointClipSpace;
	float sampleDepth;
	float rangeCheck;
	for(int i = 0; i < sampleSize; ++i)
	{
        vec3 sample = TBN * samples[i];
        sample = fragPos + sample * radius; 
		
		samplePointClipSpace = toClip(sample);
        samplePointClipSpace.xyz = samplePointClipSpace.xyz * 0.5 + 0.5; 
		
        sampleDepth = (texture(gPosition, samplePointClipSpace.xy).xyz).z;
		
		rangeCheck = smoothstep(0.0, 1.0, radius / abs((fragPos.xyz).z - sampleDepth));
		if(sampleDepth >= sample.z + bias)
			occlusion += rangeCheck;
	}  	
		
	occlusion = 1.0 - (occlusion / sampleSize);
	FragColor =  occlusion; 
}